/*
 * Copyright (C) 1996,1997 Michael R. Elkins <me@cs.hmc.edu>
 * Copyright (C) 1999-2000 Thomas Roessler <roessler@guug.de>
 * Copyright (C) 2001 Thomas Roessler <roessler@guug.de>
 *                    Oliver Ehli <elmy@acm.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#ifndef CRYPT_H
#define CRYPT_H 1

#include "mutt.h"

#if defined(HAVE_PGP) || defined(HAVE_SMIME) || defined(HAVE_GPGME)


/* For the sake of GPGME we have to define these constants unconditionally */
#define APPLICATION_PGP    (1 << 5)
#define APPLICATION_SMIME  (1 << 6)

#define ENCRYPT       (1 << 0)
#define SIGN          (1 << 1)
#define GOODSIGN      (1 << 2)
#define BADSIGN       (1 << 3)
#define SIGNOPAQUE    (1 << 4)

#define PGPENCRYPT  (APPLICATION_PGP | ENCRYPT)
#define PGPSIGN     (APPLICATION_PGP | SIGN)
#define PGPGOODSIGN (APPLICATION_PGP | GOODSIGN)
#define PGPKEY      (APPLICATION_PGP | (1 << 3))  /* fixme: What is that?*/

#define SMIMEENCRYPT  (APPLICATION_SMIME | ENCRYPT)
#define SMIMESIGN     (APPLICATION_SMIME | SIGN)
#define SMIMEGOODSIGN (APPLICATION_SMIME | GOODSIGN)
#define SMIMEBADSIGN  (APPLICATION_SMIME | BADSIGN)
#define SMIMEOPAQUE   (APPLICATION_SMIME | SIGNOPAQUE)


#define KEYFLAG_CANSIGN 		(1 <<  0)
#define KEYFLAG_CANENCRYPT 		(1 <<  1)
#define KEYFLAG_ISX509                  (1 <<  2)
#define KEYFLAG_SECRET			(1 <<  7)
#define KEYFLAG_EXPIRED 		(1 <<  8)
#define KEYFLAG_REVOKED 		(1 <<  9)
#define KEYFLAG_DISABLED 		(1 << 10)
#define KEYFLAG_SUBKEY 			(1 << 11)
#define KEYFLAG_CRITICAL 		(1 << 12)
#define KEYFLAG_PREFER_ENCRYPTION 	(1 << 13)
#define KEYFLAG_PREFER_SIGNING 		(1 << 14)

#define KEYFLAG_CANTUSE (KEYFLAG_DISABLED|KEYFLAG_REVOKED|KEYFLAG_EXPIRED)
#define KEYFLAG_RESTRICTIONS (KEYFLAG_CANTUSE|KEYFLAG_CRITICAL)

#define KEYFLAG_ABILITIES (KEYFLAG_CANSIGN|KEYFLAG_CANENCRYPT|KEYFLAG_PREFER_ENCRYPTION|KEYFLAG_PREFER_SIGNING)


/**** Generic crypto functions ****/
int mutt_protect (HEADER *, char *);

int mutt_is_multipart_encrypted (BODY *);

int mutt_is_multipart_signed (BODY *);

void mutt_signed_handler (BODY *, STATE *);

int mutt_parse_crypt_hdr (char *, int);

int crypt_query (BODY *);

void crypt_extract_keys_from_messages (HEADER *);

int crypt_get_keys (HEADER *, char **);


void crypt_forget_passphrase (void);

int crypt_valid_passphrase (int);


int crypt_write_signed(BODY *, STATE *, const char *);

void convert_to_7bit (BODY *);


/* private ? */

void crypt_current_time(STATE *, char *);

/*** gpgme functions - we can't use a gpgme prefix because this is
     used in libgpgme */
#if defined(HAVE_GPGME)

/* these are also defined in pgp.h but we are currently using one of 2
   versions: Either the one in pgp.c or the one in crypt-gpgme.c.  We
   should move a common handler for gpg and smime to crypt[-gpgme].c */
int pgp_check_traditional (FILE *, BODY *, int);
void pgp_application_pgp_handler (BODY *, STATE *);
void pgp_encrypted_handler (BODY *, STATE *);
void smime_encrypted_handler (BODY *, STATE *);
int  pgp_decrypt_mime (FILE *, FILE **, BODY *, BODY **);
int  smime_decrypt_mime (FILE *, FILE **, BODY *, BODY **);

/* These are also declared in smime.c and pgp.c - they are used here
   for crypt-gpgme.c */
int mutt_is_application_pgp (BODY *m);
int mutt_is_application_smime (BODY *m);



#include <gpgme.h>

/* We work based on user IDs, getting from a user ID to the key is
   check and does not need any memory (gpgme uses reference counting). */
struct crypt_keyinfo
{
  struct crypt_keyinfo *next;
  GpgmeKey kobj;
  int idx;             /* and the user ID at this index */
  const char *uid;     /* and for convenience point to this user ID */
  unsigned int flags;  /* global and per uid flags (for convenience)*/
};
typedef struct crypt_keyinfo crypt_key_t;


const char *crypt_keyid (crypt_key_t *k);
const char *crypt_fpr (crypt_key_t *k);
crypt_key_t *crypt_remove_key (crypt_key_t **, crypt_key_t *);
void crypt_free_key (crypt_key_t **kpp);

crypt_key_t *crypt_ask_for_key (char *tag, char *whatfor,
                                short abilities, unsigned int application);
crypt_key_t *crypt_getkeybystr (char *p, short abilities,
                                unsigned int application);
crypt_key_t *crypt_getkeybyaddr (ADDRESS * a, short abilities);


short crypt_canencrypt (unsigned char);
short crypt_cansign (unsigned char);
short crypt_get_abilities (unsigned char);

#define crypt_new_keyinfo() safe_calloc (sizeof (crypt_key_t), 1)

#endif /*HAVE_GPGME*/


/* The following globals used to be defined in pgp.h.  However we are
   using them also for the gpgme based crypto support and the
   implementations can't imclude pgp.h.  The *Command variables are
   not needed for gpgme but we better allow them otherwise we get ugly
   errors from the muttrcs */
WHERE REGEXP PgpGoodSign;

WHERE char *PgpSignAs;
WHERE short PgpTimeout;
WHERE char *PgpEntryFormat;

WHERE char *SmimeSignAs;


/* The command formats */

WHERE char *PgpClearSignCommand;
WHERE char *PgpDecodeCommand;
WHERE char *PgpVerifyCommand;
WHERE char *PgpDecryptCommand;
WHERE char *PgpSignCommand;
WHERE char *PgpEncryptSignCommand;
WHERE char *PgpEncryptOnlyCommand;
WHERE char *PgpImportCommand;
WHERE char *PgpExportCommand;
WHERE char *PgpVerifyKeyCommand;
WHERE char *PgpListSecringCommand;
WHERE char *PgpListPubringCommand;
WHERE char *PgpGetkeysCommand;




#endif /*HAVE_PGP||HAVE_SMIME||HAVE_GPGME*/
#endif /*CRYPT_H*/
